;================================================================================
;
;NXP26C92 DUAL UART (DUART) REGISTERS
;
s_c92blk =8                    ;DUART block size
n_c92ch  =2                    ;number of channels
cha_c92  =s_c92blk*0           ;channel A base
chb_c92  =s_c92blk*1           ;channel B base
;
;
;	channel A register offsets...
;
dr_mra   =cha_c92              ;ch A mode select...
;
;	xxxx0xxx > mode 0 settings
;	|||| |||
;	|||| +++> baud rate extend:  000: normal
;	||||                            001: extended mode 1
;	||||                            100: extended mode 2
;	||++> TxD FIFO IRQ level: 00: 8
;	||                               01: 4
;	||                               10: 6
;	||                               11: 1
;	|+> RxD FIFO IRQ level:  0: 1-5: see also MR1:6
;	|                                 1: 6-8: see also MR1:6
;	+> RxD watchdog:        0: disable
;	                                  1: enable
;
;	xxxxxxxx > MR1 settings
;	||||||||
;	||||||++> data format:  00: 5 bit
;	||||||                     01: 6 bit
;	||||||                     10: 7 bit
;	||||||                     11: 8 bit*
;	|||||+> parity type:   0: even*
;	|||||                       1: odd
;	|||++> parity mode:  00: check
;	|||                        01: force
;	|||                        10: don't check*
;	|||                        11: multidrop
;	||+> error mode:    0: character*
;	||                          1: block
;	|+> RxD IRQ mode:  0: RxD ready*
;	|                           1: FIFO full
;	+> RxD RTS mode:  0: off
;	                            1: on*
;	* = default setting
;
;	xxxxxxxx > mode 2 settings
;	||||||||
;	||||++++> stop bit length
;	||||
;	|||+> TxD CTS mode:        0: off
;	|||                               1: on
;	||+> TxD RTS mode:        0: off
;	||                                1: on
;	++> channel mode:       00: normal
;	                                 01: auto echo 
;	                                 10: local loop
;	                                 11: remote loop
;
dr_sra   =cha_c92+1            ;ch A status (R)...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: RxD FIFO not empty
;	||||||+> 1: RxD FIFO full
;	|||||+> 1: TxD FIFO not empty
;	||||+> 1: TxD FIFO empty
;	|||+> 1: RxD overrun
;	||+> 1: parity error
;	|+> 1: framing error
;	+> 1: break received
;
dr_csra  =dr_sra               ;ch A clock select (W)...
;
;	xxxxxxxx
;	||||||||
;	||||++++> TxD baud rate & BRG source
;	++++> RxD baud rate & BRG source
;
dr_cra   =cha_c92+2            ;ch A command (W)...
;
;	xxxxxxxx
;	||||||||
;	||||||++>   00: no operation
;	||||||         01: enable receiver
;	||||||         10: disable receiver
;	||||||         11: illegal operation
;	||||++>   00: no operation
;	||||           01: enable transmitter
;	||||           10: disable transmitter
;	||||           11: illegal operation
;	++++> 0000: no operation
;	             0001: select MR1
;	             0010: reset & disable receiver
;	             0011: reset & disable transmitter
;	             0100: reset error status
;	             0101: reset break IRQ
;	             0110: start TxD break
;	             0111: stop TxD break
;	             1000: assert RTS
;	             1001: deassert RTS
;	             1010: enable C/T timeout mode
;	             1011: select MR0
;	             1100: disable C/T timeout mode
;	             1101: undefined
;	             1110: standby power mode
;	             1111: normal power mode
;
dr_rhra  =cha_c92+3            ;ch A RxD FIFO (R)
dr_thra  =dr_rhra              ;ch A TxD FIFO (W)
;
;
;	channel B register offsets...
;
dr_mrb   =chb_c92+0            ;ch B mode select...
;
;	xxxx0000 > MR0 settings
;	||||
;	||++> TxD FIFO IRQ level: 00: 8
;	||                               01: 4
;	||                               10: 6
;	||                               11: 1
;	|+> RxD FIFO IRQ level:  0: 1-5: see also MR1:6
;	|                                 1: 6-8: see also MR1:6
;	+> RxD watchdog:        0: disable
;
;	xxxxxxxx > MR1 settings
;	||||||||
;	||||||++> data format:  00: 5 bit
;	||||||                     01: 6 bit
;	||||||                     10: 7 bit
;	||||||                     11: 8 bit*
;	|||||+> parity type:   0: even*
;	|||||                       1: odd
;	|||++> parity mode:  00: check
;	|||                        01: force
;	|||                        10: don't check*
;	|||                        11: multidrop
;	||+> error mode:    0: character*
;	||                          1: block
;	|+> RxD IRQ mode:  0: RxD ready*
;	|                           1: FIFO full
;	+> RxD RTS mode:  0: off
;	                            1: on*
;	* = default setting
;
;	xxxxxxxx > MR2 settings
;	||||||||
;	||||++++> stop bit length
;	||||
;	|||+> TxD CTS mode:  0: off
;	|||                         1: on
;	||+> TxD RTS mode:  0: off
;	||                          1: on
;	++> channel mode: 00: normal
;	                           01: auto echo 
;	                           10: local loop
;	                           11: remote loop
;
dr_srb   =chb_c92+1            ;ch B status (R)...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: RxD FIFO not empty
;	||||||+> 1: RxD FIFO full
;	|||||+> 1: TxD FIFO not empty
;	||||+> 1: TxD FIFO empty
;	|||+> 1: RxD overrun
;	||+> 1: parity error
;	|+> 1: framing error
;	+> 1: break received
;
dr_csrb  =dr_srb               ;ch B clock select (W)...
;
;	xxxxxxxx
;	||||||||
;	||||++++> TxD baud rate & BRG source
;	++++> RxD baud rate & BRG source
;
dr_crb   =chb_c92+2            ;ch B command (W)...
;
;	xxxxxxxx
;	||||||||
;	||||||++>   00: no operation
;	||||||         01: enable receiver
;	||||||         10: disable receiver
;	||||||         11: illegal operation
;	||||++>   00: no operation
;	||||           01: enable transmitter
;	||||           10: disable transmitter
;	||||           11: illegal operation
;	++++> 0000: no operation
;	             0001: select MR1
;	             0010: reset & disable receiver
;	             0011: reset & disable transmitter
;	             0100: reset error status
;	             0101: reset break IRQ
;	             0110: start TxD break
;	             0111: stop TxD break
;	             1000: assert RTS
;	             1001: deassert RTS
;	             1010: enable C/T timeout mode
;	             1011: select MR0
;	             1100: disable C/T timeout mode
;	             1101: undefined
;	             1110: undefined
;	             1111: undefined
;
dr_rhrb  =chb_c92+3            ;ch B RxD holding (R)
dr_thrb  =dr_rhrb              ;ch B TxD holding (W)
;
;
;	other register offsets...
;
dr_ipcr  =4                    ;input port change (R)
dr_acr   =dr_ipcr              ;auxiliary control register (W)...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: enable IP0 IRQ
;	||||||+> 1: enable IP1 IRQ
;	|||||+> 1: enable IP2 IRQ
;	||||+> 1: enable IP3 IRQ
;	|+++> C/T setup:
;	|
;	|            654  Mode     Source
;	|            
;	|            000  counter  IP2
;	|            001  counter  TxD Ch 1 1X clock
;	|            010  counter  TxD Ch 2 1X clock
;	|            011  counter  Xtal clock/16
;	|            100  timer    IP2
;	|            101  timer    IP2/16
;	|            110  timer    Xtal clock
;	|            111  timer    Xtal clock/16
;	|            
;	|
;	+> 0: select BRG set #1
;	             1: select BRG set #2
;
dr_imr   =5                    ;interrupt enable mask (W)...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: enable ch A THR ready
;	||||||+> 1: enable ch A RHR ready/FIFO full
;	|||||+> 1: enable ch A change in break
;	||||+> 1: enable C/T ready
;	|||+> 1: enable ch B THR ready
;	||+> 1: enable ch B RHR ready/FIFO full
;	|+> 1: enable ch B change in break
;	+> 1: enable IP0-IP3 state change
;
dr_isr   =dr_imr               ;interrupt status (R)...
;
;	xxxxxxxx
;	||||||||
;	|||||||+> 1: ch A THR ready
;	||||||+> 1: ch A RHR ready/FIFO full
;	|||||+> 1: ch A change in break
;	||||+> 1: C/T ready
;	|||+> 1: ch B THR ready
;	||+> 1: ch B RHR ready/FIFO full
;	|+> 1: ch B change in break
;	+> 1: IP0-IP3 state change
;
dr_crur  =6                    ;C/T MSB preset (W)
dr_crlr  =7                    ;C/T LSB preset (W)
dr_ctu   =dr_crur              ;C/T MSB value (R)
dr_ctl   =dr_crlr              ;C/T LSB value (R)
dr_usrdf =12                   ;user-defined (R/W)...
;
;	
;	DR_USRDF may be used for temporary storage of one byte.
;	
;
dr_opcr  =13                   ;output port configuration (W)
dr_ipsr  =dr_opcr              ;input port status (R)
dr_sctc  =14                   ;start C/T command (R)
dr_sopbc =dr_sctc              ;set output port bits command (W)
dr_rctc  =15                   ;stop C/T command (R)
dr_ropbc =dr_rctc              ;reset output port bits command (W)
;
;
;	absolute register definitions in register name order...
;
io_acia  =$000000              ;DUART base address...
;
;	
;	Set the label IO_ACIA to the base address of the 26C92.  This label must
;	be defined to a valid address.
;	
;
acr_92   =io_acia+dr_acr       ;auxiliary control (W)
ccr_92a  =io_acia+dr_cra       ;ch A command (W)
ccr_92b  =io_acia+dr_crb       ;ch B command (W)
crlr_92  =io_acia+dr_crlr      ;counter/timer LSB (W)
crur_92  =io_acia+dr_crur      ;counter/timer MSB (W)
ctlr_92  =io_acia+dr_ctl       ;counter/timer LSB (R)
ctsp_92  =io_acia+dr_rctc      ;counter/timer stop (R)
ctst_92  =io_acia+dr_sctc      ;counter/timer start (R)
ctur_92  =io_acia+dr_ctu       ;counter/timer MSB (R)
csr_92a  =io_acia+dr_csra      ;ch A clock select (W)
csr_92b  =io_acia+dr_csrb      ;ch B clock select (W)
imr_92   =io_acia+dr_imr       ;interrupt mask (W)
ipcr_92  =io_acia+dr_ipcr      ;input port change (R)
ipsr_92  =io_acia+dr_ipsr      ;input ports 0-6 status (R)
isr_92   =io_acia+dr_isr       ;interrupt status (R)
msr_92a  =io_acia+dr_mra       ;ch A mode select
msr_92b  =io_acia+dr_mrb       ;ch B mode select (W)
opcr_92  =io_acia+dr_opcr      ;output port config (W)
ropr_92  =io_acia+dr_ropbc     ;reset output port bits (W)
rxd_92a  =io_acia+dr_rhra      ;ch A RxD data in (R)
rxd_92b  =io_acia+dr_rhrb      ;ch B data in (R)
sopr_92  =io_acia+dr_sopbc     ;set output port bits (W)
sr_92a   =io_acia+dr_sra       ;ch A status (R)
sr_92b   =io_acia+dr_srb       ;ch B status (R)
txd_92a  =io_acia+dr_thra      ;ch A TxD data out (W)
txd_92b  =io_acia+dr_thrb      ;ch B data out (W)
usrdf_92 =io_acia+dr_usrdf     ;user-defined (R/W)
;
	.end
