;================================================================================
;
;iirq: HARDWARE INTERRUPT REQUEST SERVICE ROUTINE
;
;	
;	NOTE: This code services 26C92 interrupts & assumes native mode opera-
;	      tion.  Code unrelated to 26C92 IRQs may be added at IIRQB.  More
;	      commentary on this follows.
;	
;
iirq     phb                   ;save DB
         phd                   ;save DP
         longr                 ;16 bit registers
         pha                   ;save .C
         phx                   ;save .X
         phy                   ;save .Y
;
;
;Local Stack Frame Definitions
;
irq_yrx  .= 1                  ;.Y
irq_xrx  .= irq_yrx+s_word     ;.X
irq_arx  .= irq_xrx+s_word     ;.A
irq_dpx  .= irq_arx+s_word     ;DP
irq_dbx  .= irq_dpx+s_mpudpx   ;DB
irq_srx  .= irq_dbx+s_mpudbx   ;SR
irq_pcx  .= irq_srx+s_mpusrx   ;PC
irq_pbx  .= irq_pcx+s_mpupcx   ;PB
;
;
;         jmp (ivirq)           ;IRQ indirect vector...
;
;	
;	If your system has an indirect vector for IRQ processing (recommended),
;	it should point to the IIRQA label immediately below.  The IIRQ label
;	at the start of this ISR would be referred to by the 65C816 IRQ hardware
;	vector at $00FFEE-$00FFEF.
;	
;
iirqa    longa                 ;ensure 16 bit accumulator
         shortr                ;8 bit registers
;
;	
;	IRQ priority: a) ch A receive
;	              b) ch B receive
;	              c) ch A transmit
;	              d) ch B transmit
;	
;
         lda isr_92            ;DUART interrupt status
         bne .iirq010          ;DUART is interrupting
;
         jmp iirqb             ;DUART is not interrupting...
;
;	
;	If more ISR code is needed it should be added at IIRQB, which currently
;	is a placeholder & falls through to the common interrupt return (CRTI)
;	code.  Generally speaking, servicing of I/O IRQs should have high prior-
;	ity to minimize the likelihood of data under-/overrun.
;	
;
.iirq010 bit #u92arirq         ;ch A receiver interrupting?
         beq .iirq040          ;no
;
;
;	process channel A RxD IRQ...
;
.iirq020 lda rxd_92a           ;read datum from FIFO &...
         xba                   ;hold it
         ldx e32rixpa          ;RxD buffer "put" index
         txa                   ;determine...
         ina                   ;next position
         and #e32xmask         ;wrap index to buffer size
         cmp e32rixga          ;check RxD buffer "get" index
         beq .iirq030          ;buffer full, drop datum...
;
;	
;	If the incoming datum can't be buffered then there is no choice but to
;	drop it.  This will only happen if the remote device is continuously
;	transmitting & nothing at this end is reading data from the buffer.
;	However, the 26C92 may automatically stop the remote device if full
;	hardware handshaking is implemented, thus preventing a data overrun.
;	
;
         xba                   ;swap datum w/index &...
         sta e32bufra,x        ;buffer it
         xba                   ;get back the index &...
         sta e32rixpa          ;store it
;
.iirq030 lda sr_92a            ;ch A status
         bit #u92rxdr          ;more data in FIFO?
         bne .iirq020          ;yes...
;
;	
;	The loop back to .IIRQ20 will repeat until the FIFO has been completely
;	emptied.  Hence as many as 8 incoming bytes can be processed in a single
;	interrupt, which will greatly reduce the IRQ load on the MPU.
;	
;
         lda isr_92            ;reload interrupt status
;
.iirq040 bit #u92brirq         ;ch B receiver interrupting?
         beq .iirq070          ;no
;
;
;	process channel B RxD IRQ...
;
;	
;	The following code works the same as for channel A.  Read the above com-
;	ments, as they apply here as well.
;	
;
.iirq050 lda rxd_92b
         xba
         ldx e32rixpb
         txa
         ina
         and #e32xmask
         cmp e32rixgb
         beq .iirq060
;
         xba
         sta e32bufrb,x
         xba
         sta e32rixpb
;
.iirq060 lda sr_92b
         bit #u92rxdr
         bne .iirq050
;
         lda isr_92            ;reload interrupt status
;
.iirq070 bit #u92atirq         ;ch A transmitter interrupting?
         beq .iirq110          ;no, goto ch B
;
;
;	process channel A TxD IRQ...
;
         ldx e32tixga          ;buffer "get" index
;
.iirq080 cpx e32tixpa          ;buffer "put" index
         beq .iirq090          ;buffer empty...
;
;	
;	If the TxD buffer is empty then we shut down the transmitter so it won't
;	keep bugging the MPU with interrupts.   Otherwise, we attempt to process
;	the buffer.
;	
;
         lda sr_92a            ;channel A status
         bit #u92txdr          ;TxD FIFO full?
         beq .iirq100          ;yes, defer next datum...
;
;	
;	If the FIFO is full then we cannot pass anymore bytes to the DUART.  So
;	we skip ahead & wait until the next TxD IRQ, at which time room will be-
;	come available in the FIFO.  See commentary in the file FOREGROUND.ASM
;	concerning transmit blocking.
;	
;
         lda e32bufta,x        ;get datum from buffer &...
         sta txd_92a           ;write to FIFO
         txa                   ;current buffer position
         ina                   ;new buffer position
         and #e32xmask         ;deal with wrap &...
         sta e32tixga          ;set next position
         tax
         bra .iirq080          ;try to send another byte...
;
;	
;	The loop back to .IIRQ080 will repeat until the TxD FIFO has been comp-
;	pletely filled.  Hence as many as 8 outgoing bytes can be processed in
;	a single interrupt, which will greatly reduce the IRQ load on the MPU.
;	
;
.iirq090 lda #u92crtxd         ;disable xmitter so...
         sta ccr_92a           ;it quits interrupting
         lda #e32txsta         ;tell foreground that...
         tsb e32txst           ;xmitter is disabled...
;
;	
;	When a call is made to PUTCHA the datum is buffered (if possible) & then
;	the E32TXST (transmitter status) flag is tested to see if the transmit-
;	ter has been disabled.  If it has, PUTCHA will enable the transmitter,
;	which will immediately cause a FIFO empty IRQ to occur.  Hence transmit-
;	ter control is automatic in nature.
;
;	There is a potential "gotcha" with this arrangement.  If the transmitter
;	has been disabled but the associated flag bit in E32TXST is inadvertent-
;	ly cleared (e.g., due to a programming error that overwrites E32TXST),
;	transmission will cease, as GETCHA will assume that the transmitter is
;	already running & won't restart it.
;	
;
.iirq100 lda isr_92            ;reload ISR
;
         bit #u92btirq         ;ch B transmitter interrupting?
         beq iirqb             ;no, done with DUART
;
;	
;	The following code works the same as for channel A.  Read the above com-
;	ments, as they apply here as well.
;	
;
         ldx e32tixgb
;
.iirq110 cpx e32tixpb
         beq .iirq120
;
         lda sr_92b
         bit #u92txdr
         beq iirqb
;
         lda e32buftb,x
         sta txd_92b
         txa
         ina
         and #e32xmask
         sta e32tixgb
         tax
         bra .iirq110
;
.iirq120 lda #u92crtxd
         sta ccr_92b
         lda #e32txstb
         tsb e32txst
;
iirqb    ;patch additional ISR code here
;
;================================================================================
;
;crti: COMMON INTERRUPT RETURN
;
crti     longr                 ;16 bit registers
         ply                   ;restore Y
         plx                   ;restore X
         pla                   ;restore .C
         pld                   ;restore DP
         plb                   ;restore DB
         rti                   ;resume foreground processing
;
	.end
